"use strict";

import React, {Component} from 'react';
import {
    createReactNavigationReduxMiddleware,
    createReduxBoundAddListener,
    createNavigationReducer
} from 'react-navigation-redux-helpers';
import {connect} from 'react-redux';

export const navigatorPieces = (BareNavigator, name = "root") => {
    const middleware = createReactNavigationReduxMiddleware(name, x => x);
    const addListener = createReduxBoundAddListener(name);
    const reducer = createNavigationReducer(BareNavigator);

    class Navigator extends Component {
        render () {
            const {dispatch, navigationState} = this.props;
            return <BareNavigator
                navigation={{dispatch, state: navigationState, addListener}}
            />;
        }
    }

    return {Navigator, reducer, middleware};
};

export const connectNavigator = selectorFn => connect(state => ({navigationState: selectorFn(state)}));

import {NavigationActions, StackActions} from 'react-navigation';

const {navigate, back} = NavigationActions;
const {reset} = StackActions;

export const backButtonHandler = ({dispatch, getState}) => {
    if (getState().index === 0) return false;
    dispatch(back());
    return true;
};

const topOfNavigationStack = ({routes}) => routes[routes.length - 1] || {};
const isRoute = expected => ({routeName}) => routeName === expected;
const isTopRoute = expected => state => isRoute(expected)(topOfNavigationStack(state));

export const navigationHelpers = {
    topOfNavigationStack,

    resetRoutes: routeNames => reset({
        actions: routeNames.map(routeName => navigate({routeName})),
        index: routeNames.length - 1
    }),

    isRoute,

    isTopRoute,

    goBackFrom: routeName => ({dispatch, getState}) => {
        if (isTopRoute(routeName)(getState())) {
            dispatch(back());
        }
    },
};
